#!/bin/sh
# *************************************************************************
#
# Licensed Materials - Property of IBM
#
# 5724-L62
#
# (C) Copyright IBM Corp. 2006  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *************************************************************************
# @(#)22  1.4.1.1  kyn/java/etc/kyncollect.sh, am_itm.kyn, itm_war_62 9/14/07 09:19:00
# *************************************************************************

PCL=ht
PCU=HT
SCRIPT=`basename $0`

print_help()
{
  cat << EOF

        $SCRIPT                Version 06.20.04


This program is intended to collect diagnostic information from the
computer upon which it is executed. Collected data includes:
        - system configuration;
        - environment;
        - disk space information;
        - network state;
        - software revision;
        - Monitoring Agent configuration and log files;
        - IPC facilities status;
        - monitored webserver (Apache, IHS) configuration and log files.

Command line syntax is:

        $SCRIPT

EOF

  exit 0
}

report_error()
{
  exec 1>&2
  echo
  echo Error: $1
  echo
  exit 1
}

if [ -n "$1" ]; then
  if [ "$1" = "-help" ]; then
    print_help
  else
    report_error "unknown option  $1"
  fi
fi

PROD_HOME=`dirname $0`
test -z "$PROD_HOME" && PROD_HOME=.
cd $PROD_HOME/..
PROD_HOME=`pwd`

PATH="/usr/bin:/usr/sbin:/usr/ucb:$PATH"
export PATH

TmpDir=$HOME/tmp/k"$PCL"
DestName=k"$PCL"collect-`hostname`
DigDir=$TmpDir/$DestName

rm -Rf $DigDir $DigDir.tar* 2>/dev/null
mkdir -p $DigDir || exit 1
cd $DigDir || exit 1


echo "...Collecting system configuration information"

OSVersion=`uname -r`
OSType=`uname -s`
OSArch=`uname -m`

PROC_INFO=
TOP=
SHOWREV=
DMESG=
NETSTAT_OPT="-a -n"

WHOAMI=`whoami 2>/dev/null`
[ -z "$WHOAMI" ] && WHOAMI=`id | cut -d"(" -f2 | cut -d")" -f1`


case "$OSType" in
  HP-UX)            
    OSBits=`getconf KERNEL_BITS 2>&1`
    TOP="top -d1 -u -h"
    SHOWREV="swlist -v"
  ;;

  AIX)
    OSVersion=`lslpp -L bos.rte.boot | grep Boot`
    OSVersion=`echo $OSVersion | cut -f 2 -d " "`
    test "$OSVersion" || OSVersion=`oslevel`

    OSBits=`bootinfo -K 2>/dev/null`
    test "$OSBits" || OSBits=`getconf KERNEL_BITMODE 2>&1`

    if [ -z "$OSBits" ]; then
      Kernel=`lsconf -k 2>/dev/null`
      test "$Kernel" && OSBits=`echo "$Kernel" | cut -d " " -f3 | cut -c-2 2>/dev/null`
    fi

    PROC_INFO=`/usr/sbin/lsdev -S A -c processor 2>/dev/null`
    SHOWREV="lslpp -hav"
  ;;

  Linux)
    case "$OSArch" in
      s390x|ppc64|x86_64) OSBits=64 ;;
      *) OSBits=32 ;;
    esac

    test -r /proc/cpuinfo && PROC_INFO=`cat /proc/cpuinfo`

    DMESG=dmesg
    TOP="top b n1"
    NETSTAT_OPT="$NETSTAT_OPT -p"
    SHOWREV="rpm -qaiv"
  ;;

  SunOS)
    OSBits=`isainfo -b 2> /dev/null`
    PROC_INFO=`psrinfo && echo && isainfo -vk && isainfo -v`
    TOP="prstat 5 1"
    SHOWREV="showrev -a"
    DMESG=dmesg
  ;;

esac


cat > system.info << EOF
OS type     : $OSType 
OS version  : $OSVersion
OS bits     : $OSBits

User        : $WHOAMI
Group       : `groups 2>&1`
Host name   : `hostname 2>&1`
Domain name : `domainname 2>&1`
Date        : `date 2>&1`
Runtime     : $PROD_HOME

==> uname:
`uname -a 2>&1`

==> ulimit:
`ulimit -a 2>&1`
EOF

if [ -n "$PROC_INFO" ]; then
cat >> system.info << EOF

==> Processor(s) :
$PROC_INFO
EOF
fi

case "$OSType" in
  Linux)
    echo >> system.info
    echo "==> Memory:" >> system.info
    free >> system.info
    echo >> system.info

    if [ -r /proc/net/sockstat ]; then
      echo "==> Socket state:" >> system.info
      cat /proc/net/sockstat >> system.info
    fi

    RELS="redhat-release SuSE-release"

    for R in $RELS; do
      test -r /etc/$R && cp -f /etc/$R .
    done

    vmstat > vmstat.info 2>&1
  ;;

  AIX)
    instfix -i > instfix.info 2>&1
    vmstat -sv > vmstat.info 2>&1
    test -x /usr/bin/svmon && /usr/bin/svmon -G > svmon.info 2>&1
    test -x /usr/bin/lparstat && /usr/bin/lparstat -i > lparstat.info 2>&1
    test -x /usr/sbin/swap && /usr/sbin/swap -l > swap.info 2>&1
    test -x /usr/sbin/prtconf && /usr/sbin/prtconf >> system.info 2>&1
  ;;
esac

cat >> system.info << EOM

==> IPC status:
`ipcs -a 2>&1`

EOM

echo "...Collecting environment information"
ls -laRL $PROD_HOME/../.. >> dir.info
env > env.info 2>&1

echo "...Collecting disk space information"
df -k > disk.info 2>&1

echo "...Collecting network state"
netstat $NETSTAT_OPT > netstat.info 2>&1
cp -f /etc/hosts hosts.info 2>/dev/null

if [ -n "$TOP" ]; then
  echo "...Collecting system load information"
  $TOP > top.info 2>&1
fi

if [ -n "$DMESG" ]; then
  echo "...Collecting kernel ring buffer"
  $DMESG > dmesg.info 2>&1
fi

if [ -n "$SHOWREV" ]; then
  echo "...Collecting software revision information"
  $SHOWREV > rev.info 2>&1
fi

echo "...Collecting log files"
mkdir logs || exit 1
exec 2>/dev/null
cp -fp $PROD_HOME/../logs/*"$PCU"*.log logs/ 
cp -fp $PROD_HOME/../logs/*"$PCL"*.log logs/ 
cp -fp $PROD_HOME/../logs/kju*.log logs/
cp -fp $PROD_HOME/../../logs/*"$PCU"* logs/ 
cp -fp $PROD_HOME/../../logs/*"$PCL"* logs/ 
cp -fp $PROD_HOME/../../logs/*nstall*.log logs/ 
cp -fp $PROD_HOME/../../logs/*andle*.log logs/ 
cp -fp $PROD_HOME/../../logs/db2*.log logs/ 
cp -fp $PROD_HOME/javacore* logs/ 
cp -fp $PROD_HOME/javadump* logs/ 
exec 2>&1

echo "...Collecting configuration and registry files"
exec 2>/dev/null
cp -rfp $PROD_HOME/../../config .
cp -rfp $PROD_HOME/../../registry .
cp -rfp $PROD_HOME/config ht_config
cp -rfp $PROD_HOME/../wj/config wj_config
exec 2>&1

echo "...Collecting Apache configuration and log files"
apDirValue=""

apache_directive()
{
#    local cfg dirName dirValue

    apDirValue=""
    cfg=$1
    dirName=$2
    dirValue=`grep "^[ \t]*$dirName" $cfg | cut -d " " -f 2 | tr -d \"`
    
    if [ -z "$dirValue" ]; then
        echo "Error: cannot find $dirName directive in $cfg"
        return 1
    fi
    
    apDirValue=$dirValue
}

apache_entry()
{
#    local sectName cfg dest srvRoot errorLog
    
    sectName=$1
    cfg=$2
    echo "Apache section $sectName with config file $cfg"

    if [ ! -r $cfg ]; then
        echo "Error: file $cfg does not exist"
        return 1
    fi

    dest="apache/$sectName"
    
    echo "  ... collecting apache configuration files"
    mkdir -p $dest/conf
    cp -fp $cfg $dest/conf
    cp -fp `dirname $cfg`/*kht* $dest/conf 2>/dev/null

    echo "  ... collecting apache error logs"
    mkdir -p $dest/logs
    
    apache_directive $cfg "ServerRoot"
    if [ -z "$apDirValue" ]; then
        return 1
    else
        srvRoot=$apDirValue
    fi

    apache_directive $cfg "ErrorLog"
    if [ -z "$apDirValue" ]; then
        return 1
    else
        errorLog=$apDirValue
    fi

    if [ -r $errorLog ]; then
        cp -fp $errorLog* $dest/logs 2>/dev/null
    elif [ -r $srvRoot/$errorLog ]; then
        cp -fp $srvRoot/$errorLog* $dest/logs 2>/dev/null
    else
        echo "Warn: not error_log files collected"
    fi
}

ht_parse_config()
{
    echo "Processing config file $1"
    tr '\t]' ' \n' < $1\
      | sed -e 's/SECTION/@SECTION/g' \
      | tr @ '\n'\
      | sed -e '/Apache/!d'\
            -e 's/^.*Apache_Alias_Config:\([^ ]*\) .*KHT_APACHE_WEBSRVR_CONFIG_PATH=\([^}]*\).*$/apache_entry \1 \2/'\
      | while read i; do eval $i; done
}

ht_parse_config $PROD_HOME/../../config/*_ht.cfg


echo "...Packing collected information"
cd ..
tar -cf $DestName.tar $DestName || exit 1
gzip -9 $DestName.tar || exit 1
rm -Rf $DestName

echo "
********************************************************************
$DigDir.tar.gz
This file contains diagnostic information.
Forward this binary file to your IBM Support representative.
********************************************************************
"
